@@ -2,19 +2,22 @@
 #'
 #' @description This function will provide the best combinations of normalization
 #' and imputation methods for the user given dataset based on the intragroup
-#' variation evaluation parameters called PCV, PEV and PMAD.
+#' variation evaluation parameters called PCV, PEV, PMAD and NRMSE.
 #'
 #' @param data_input Label-free proteomics expression data as a dataframe
 #' @param groups Group information about the input data
+#' @param data_type A character string specifying the type of data being used. Use "Peptide" if your dataset contains peptide information, where the first column represents peptide IDs and the second column represents protein IDs. Use "Protein" if your dataset consists of protein data, where the first column represents protein IDs. The function will handle the data accordingly based on this parameter.
+#' @param aggr_method A character string specifying the method for aggregating peptide data to corresponding protein values. Use "sum" to aggregate by the sum of peptide intensities, "mean" to aggregate by the mean of peptide intensities, or "median" to aggregate by the median of peptide intensities. This parameter is only applicable when "data_type" is set to "Peptide".
 #'
 #' @details Label-free LC-MS proteomics expression data is often affected by heterogeneity and missing values. 
 #' Normalization and missing value imputation are the commonly used techniques to solve these issues and make the dataset suitable for further downstream analysis. 
 #' This function provides the best combination of normalization and imputation methods for the dataset, choosing from the three normalization methods (vsn, loess, and rlr) and three imputation methods (knn, lls, svd). 
 #' The intragroup variation evaluation measures named pooled co-efficient of variance (PCV), pooled estimate of variance (PEV) and pooled median absolute deviation (PMAD) are used for selecting the best combination of normalization and imputation method for the given dataset.
-#' It will return the best combinations based on each evaluation parameters of
-#' PCV, PEV, and PMAD.
+#' It will return the best combinations based on each evaluation parameters of PCV, PEV, and PMAD. 
 #'
-#' Along with this, the user can get all three normalized datasets, nine combinations of normalized and missing values imputed datasets, and the PCV, PEV, and PMAD result values.
+#' Along with this, the user can get all three normalized datasets, nine combinations of normalized and missing values imputed datasets, and the PCV, PEV, and PMAD result values. The user can also obtain the Normalized Root Mean Square Error (NRMSE) values, 
+#' calculated by comparing the normalized and imputed dataset with the original dataset across all nine combinations.
+#' These NRMSE values provide insight into the accuracy of the imputation and normalization processes.
 #'
 #' @returns
 #'  This function gives the list  which consist of following results.
@@ -31,6 +34,12 @@
 #' `PMAD Result` Values of groupwise PMAD, overall PMAD, PMAD mean, PMAD median
 #'               and PMAD standard deviation for all combinations.
 #'
+#' `NRMSE Result` NRMSE values calculated for the normalized and imputed dataset 
+#'                to the original dataset. 
+#'
+#' `rollup_protein` The aggregated protein values for the peptide dataset are 
+#'                  based on either the sum, mean, or median.
+#'                
 #' `vsn_data` The `vsn` normalized dataset
 #'
 #' `loess_data` The `loess` normalized dataset
@@ -73,7 +82,7 @@
 #'
 #' @examples
 #' \donttest{
-#' result <- best_combination(yeast_data, yeast_groups)
+#' result <- best_combination(yeast_data, yeast_groups, data_type = "Protein")
 #' result$`Best combinations`
 #' result$`PCV Result`
 #' result$`PMAD Result`
@@ -81,64 +90,101 @@
 #' }
 
 #Main function for finding out the top three combinations
-best_combination <- function (data_input, groups){
+best_combination <- function (data_input, groups, data_type, aggr_method){
   
   sink("output.txt")
-
+  
   Type <- Group <- name <- value <- . <- value_Mean <- median <-
     value_Median <- sd <- value_SD <- PCV_mean <- PCV_median <- PCV_sd <-
     PEV_mean <- PEV_median <- PEV_sd <- PMAD_mean <- PMAD_median <- PMAD_sd <-
-    rowid <- original_order <- group <- mass <- all_na <- NULL
+    rowid <- original_order <- group <- mass <- all_na <- where <- type <-
+    val <- pair_num <- grp <- vals<- NULL
+  
+  # Adding peptide to protein aggregation functionality
+  aggregate_peptide_to_protein <- function(data, method) {
+    # Ensure the data has at least three columns: Peptide ID, Protein ID, and expression values
+    if (ncol(data) < 3) {
+      stop("Peptide data must have at least three columns (Peptide ID, Protein ID, and expression values).")
+    }
+    
+    peptide_column <- names(data)[1]  # First column is assumed to be Peptide ID
+    protein_column <- names(data)[2]  # Second column is assumed to be Protein ID
+    
+    # Check if the aggregation method is valid
+    if (!method %in% c("sum", "mean", "median")) {
+      stop("Invalid aggregation method. Choose either 'sum', 'mean', or 'median'.")
+    }
+    
+    # Convert column names to symbols for dplyr functions
+    protein_col_sym <- dplyr::sym(protein_column)
+    
+    # Perform aggregation based on the Protein ID column
+    aggregated_data <- data %>%
+      dplyr::group_by(!!protein_col_sym) %>%
+      dplyr::summarise(dplyr::across(where(is.numeric), ~ match.fun(method)(., na.rm = TRUE)))  # Aggregate numeric columns
+    
+    # Rename the Protein ID column back to original after aggregation
+    colnames(aggregated_data)[1] <- protein_column
+    
+    return(aggregated_data)
+  }
+  
+  # If peptide data is chosen, perform aggregation, otherwise proceed with the protein data
+  if (data_type == "Peptide" && !is.null(aggr_method)) {
+    data_input <- aggregate_peptide_to_protein(data_input, aggr_method)
+  }
   
   #Converting all zeros to NAs
   data_input[data_input == 0] <- NA
-
- complete_data_fn <- function(data, groups) {
-  # Rename the columns in the groups data
-  colnames(groups) <- c("name", "group")
-   
-  # Add a new column to preserve the original order of rows
-  data <- data %>% dplyr::mutate(original_order = dplyr::row_number())
-   
-  # Rename the first column of data
-  colnames(data)[1] <- "rowid"
-   
-  # Reshape the data into long format
-  long_data <- data %>%
-   tidyr::pivot_longer(-c(rowid, original_order), names_to = "name", values_to = "mass")
-   
-  # Merge with groups to add group information
-  long_data <- long_data %>%
-    dplyr::left_join(groups, by = "name")
-   
-  # Identify rows where any group has all missing values
-  group_summary <- long_data %>%
-    dplyr::group_by(rowid, group) %>%
-    dplyr::summarise(all_na = all(is.na(mass)), .groups = 'drop') %>%
-    dplyr::ungroup()
-   
-  # Identify rows to remove (where any group has all missing values)
-  rows_to_remove <- group_summary %>%
-    dplyr::group_by(rowid) %>%
-    dplyr::summarise(remove = any(all_na)) %>%
-    dplyr::filter(remove) %>%
-    dplyr::pull(rowid) %>%
-   unique()
-   
-  # Filter out rows with any completely missing group
-  filtered_data <- long_data %>%
-    dplyr::filter(!(rowid %in% rows_to_remove)) %>%
-    dplyr::select(-group)
-   
-  # Reshape back to wide format and reorder based on the original order
-  com_data <- filtered_data %>%
-   tidyr::pivot_wider(names_from = name, values_from = mass) %>%
-    dplyr::arrange(original_order) %>%
-    dplyr::select(-original_order)
-   
-  return(com_data)
- } 
- 
+  #save it for output
+  rollup_protein <- data_input
+  
+  complete_data_fn <- function(data, groups) {
+    # Rename the columns in the groups data
+    colnames(groups) <- c("name", "group")
+    
+    # Add a new column to preserve the original order of rows
+    data <- data %>% dplyr::mutate(original_order = dplyr::row_number())
+    
+    # Rename the first column of data
+    colnames(data)[1] <- "rowid"
+    
+    # Reshape the data into long format
+    long_data <- data %>%
+      tidyr::pivot_longer(-c(rowid, original_order), names_to = "name", values_to = "mass")
+    
+    # Merge with groups to add group information
+    long_data <- long_data %>%
+      dplyr::left_join(groups, by = "name")
+    
+    # Identify rows where any group has all missing values
+    group_summary <- long_data %>%
+      dplyr::group_by(rowid, group) %>%
+      dplyr::summarise(all_na = all(is.na(mass)), .groups = 'drop') %>%
+      dplyr::ungroup()
+    
+    # Identify rows to remove (where any group has all missing values)
+    rows_to_remove <- group_summary %>%
+      dplyr::group_by(rowid) %>%
+      dplyr::summarise(remove = any(all_na)) %>%
+      dplyr::filter(remove) %>%
+      dplyr::pull(rowid) %>%
+      unique()
+    
+    # Filter out rows with any completely missing group
+    filtered_data <- long_data %>%
+      dplyr::filter(!(rowid %in% rows_to_remove)) %>%
+      dplyr::select(-group)
+    
+    # Reshape back to wide format and reorder based on the original order
+    com_data <- filtered_data %>%
+      tidyr::pivot_wider(names_from = name, values_from = mass) %>%
+      dplyr::arrange(original_order) %>%
+      dplyr::select(-original_order)
+    
+    return(com_data)
+  } 
+  
   com_data <- complete_data_fn (data_input, groups)
   
   #Giving original name to the first column
@@ -171,8 +217,22 @@ best_combination <- function (data_input, groups){
     return(grouped_data)
   }
   
+  # Function to replace NaN with NA in list elements
+  replace_NaN_with_NA <- function(data_list) {
+    lapply(data_list, function(df) {
+      df[] <- lapply(df, function(col) {
+        if (is.numeric(col)) {
+          col[is.nan(col)] <- NA
+        }
+        return(col)
+      })
+      return(df)
+    })
+  }
+  
   #Grouping of dataframe as a triplicate groups
-  group_data <- grouping_data(com_data2, groups)
+  group_data <- grouping_data(com_data2, groups) 
+  group_data <- replace_NaN_with_NA(group_data)
   
   #VSN Normalization function
   VSN_Norm <- function(dat) {
@@ -240,25 +300,47 @@ best_combination <- function (data_input, groups){
   rlr_group_data <- grouping_data(rlr.dat, groups)
   
   #Imputation of normalized datasets
-  #KNN imputation
-  KNN_Imputation <- function (dat)
-  {
+  KNN_Imputation <- function(dat) {
+    # Check if there are any missing values in the dataset
+    if (!anyNA(dat)) {
+      # If no missing values, return the dataset as is
+      message("No missing values found. Returning original dataset.")
+      return(dat)
+    }
+    
+    # If there are missing values, perform KNN imputation
     resultkNN <- VIM::kNN(dat, numFun = laeken::weightedMean, weightDist = TRUE,
-                          imp_var = FALSE, k= 10)
+                          imp_var = FALSE, k = 10)
     return(resultkNN)
   }
   
+  
   #LLS imputation
-  LLS_Imputation <- function (dat)
-  {
-    resultLLS <- pcaMethods::llsImpute(dat, k=2, correlation = "pearson", allVariables = TRUE)
+  LLS_Imputation <- function(dat) {
+    # Check if there are any missing values in the dataset
+    if (!anyNA(dat)) {
+      # If no missing values, return the dataset as is
+      message("No missing values found. Returning original dataset.")
+      return(dat)
+    }
+    
+    # If there are missing values, perform LLS imputation
+    resultLLS <- pcaMethods::llsImpute(dat, k = 2, correlation = "pearson", allVariables = TRUE)
     dataSet.imputed <- resultLLS@completeObs
     return(dataSet.imputed)
   }
   
+  
   #SVD imputation
-  SVD_Imputation <- function (dat)
-  {
+  SVD_Imputation <- function(dat) {
+    # Check if there are any missing values in the dataset
+    if (!anyNA(dat)) {
+      # If no missing values, return the dataset as is
+      message("No missing values found. Returning original dataset.")
+      return(dat)
+    }
+    
+    # If there are missing values, perform SVD imputation
     resultSVD <- pcaMethods::pca(dat, method = "svdImpute", nPcs = 2)
     dataSet.imputed <- resultSVD@completeObs
     return(dataSet.imputed)
@@ -396,13 +478,34 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result
-  final_Group_data_PCV_mean <- subset(total_Group_data_PCV_mean2, select = -row)
+  grouping_result <- function (data){
+    result2 <- as.data.frame(data |>
+                               dplyr::mutate(row = dplyr::row_number()) |>
+                               tidyr::pivot_longer(-row, values_transform = as.character) |>
+                               dplyr::mutate(pair_num = (dplyr::row_number() + 1) %/% 2, 
+                                             type = dplyr::if_else(dplyr::row_number() %% 2 == 1, "val", "grp"), .by = row) |>
+                               dplyr::select(-name) |>
+                               tidyr::pivot_wider(names_from = type, values_from = value) |>
+                               dplyr::summarize(vals = paste0(val, collapse = ", "),
+                                                .by = c(pair_num, grp)) |>
+                               dplyr::mutate(row = dplyr::row_number(), .by = pair_num) |>
+                               tidyr::pivot_wider(names_from = pair_num, values_from = c(vals, grp), names_vary = "slowest") |>
+                               dplyr::select(-row) |>
+                               `colnames<-`(colnames(data)))
+    
+    result <- result2[1,]
+    return(result)
+  } 
+  
+  final_Group_data_PCV_mean1 <- subset(total_Group_data_PCV_mean2, select = -row)
+  
+  final_Group_data_PCV_mean <- grouping_result(final_Group_data_PCV_mean1)
   
   ###PCV_median
   #Combining all the above results
   total_pcv_median <- plyr::rbind.fill(vsn_knn_PCV_median, vsn_lls_PCV_median, vsn_svd_PCV_median, 
-                                     loess_knn_PCV_median, loess_lls_PCV_median, loess_svd_PCV_median, 
-                                     rlr_knn_PCV_median, rlr_lls_PCV_median, rlr_svd_PCV_median)
+                                       loess_knn_PCV_median, loess_lls_PCV_median, loess_svd_PCV_median, 
+                                       rlr_knn_PCV_median, rlr_lls_PCV_median, rlr_svd_PCV_median)
   
   #Separating the results groupwise
   total_Group_data_PCV_median <- total_pcv_median %>%
@@ -423,13 +526,15 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result 
-  final_Group_data_PCV_median <- subset(total_Group_data_PCV_median2, select = -row)
+  final_Group_data_PCV_median1 <- subset(total_Group_data_PCV_median2, select = -row)
+  
+  final_Group_data_PCV_median <- grouping_result (final_Group_data_PCV_median1)
   
   ###PCV_sd
   #Combining all the above results
   total_pcv_sd <- plyr::rbind.fill(vsn_knn_PCV_sd, vsn_lls_PCV_sd, vsn_svd_PCV_sd, 
-                                     loess_knn_PCV_sd, loess_lls_PCV_sd, loess_svd_PCV_sd, 
-                                     rlr_knn_PCV_sd, rlr_lls_PCV_sd, rlr_svd_PCV_sd)
+                                   loess_knn_PCV_sd, loess_lls_PCV_sd, loess_svd_PCV_sd, 
+                                   rlr_knn_PCV_sd, rlr_lls_PCV_sd, rlr_svd_PCV_sd)
   
   #Separating the results groupwise
   total_Group_data_PCV_sd <- total_pcv_sd %>%
@@ -450,7 +555,9 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result
-  final_Group_data_PCV_sd <- subset(total_Group_data_PCV_sd2, select = -row)
+  final_Group_data_PCV_sd1 <- subset(total_Group_data_PCV_sd2, select = -row)
+  
+  final_Group_data_PCV_sd <- grouping_result (final_Group_data_PCV_sd1)
   
   #Overall data PCV
   Total_data_PCV = function(data){
@@ -498,7 +605,8 @@ best_combination <- function (data_input, groups){
   total_pcv_overall_mean2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pcv_overall_mean <-total_pcv_overall_mean2%>%dplyr::slice_min(PCV_mean, n=1, with_ties = TRUE)
+  total_pcv_overall_mean1 <-total_pcv_overall_mean2%>%dplyr::slice_min(PCV_mean, n=1, with_ties = TRUE)
+  total_pcv_overall_mean <- grouping_result(total_pcv_overall_mean1)
   original_cols <- c("Overall_Type.PCV_mean", "Overall_value.PCV_mean")
   colnames(total_pcv_overall_mean) <- original_cols
   
@@ -534,7 +642,8 @@ best_combination <- function (data_input, groups){
   total_pcv_overall_median2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pcv_overall_median <-total_pcv_overall_median2%>%dplyr::slice_min(PCV_median, n=1, with_ties = TRUE)
+  total_pcv_overall_median1 <-total_pcv_overall_median2%>%dplyr::slice_min(PCV_median, n=1, with_ties = TRUE)
+  total_pcv_overall_median <- grouping_result(total_pcv_overall_median1)
   original_cols <- c("Overall_Type.PCV_median", "Overall_value.PCV_median")
   colnames(total_pcv_overall_median) <- original_cols
   
@@ -570,7 +679,8 @@ best_combination <- function (data_input, groups){
   total_pcv_overall_sd2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pcv_overall_sd <-total_pcv_overall_sd2%>%dplyr::slice_min(PCV_sd, n=1, with_ties = TRUE)
+  total_pcv_overall_sd1 <-total_pcv_overall_sd2%>%dplyr::slice_min(PCV_sd, n=1, with_ties = TRUE)
+  total_pcv_overall_sd <- grouping_result(total_pcv_overall_sd1)
   original_cols <- c("Overall_Type.PCV_sd", "Overall_value.PCV_sd")
   colnames(total_pcv_overall_sd) <- original_cols
   
@@ -587,13 +697,20 @@ best_combination <- function (data_input, groups){
   # Remove even-numbered columns
   result_PCV_names1 <- result_PCV[, -col_indices]
   n <- matrix(t(result_PCV_names1), ncol=1)
-  all_elements <- unlist(n)
+  # Split the values by comma and remove extra spaces
+  split_data <- strsplit(n, ",\\s*")
+  
+  # Flatten the list into a single vector
+  separated_data <- unlist(split_data)
   
   # Get the frequency of each element in the dataframe
-  freq <- table(all_elements)
+  freq <- table(separated_data)
   
-  # Find the most occurring element
-  PCV_best_combination <- names(freq)[which.max(freq)]
+  # Find the maximum frequency
+  max_freq <- max(freq)
+  
+  # Get the names of all elements that have the maximum frequency
+  PCV_best_combination <- names(freq)[freq == max_freq]
   
   #Groupwise PEV estimation
   Group_data_PEV = function(data1, groups){
@@ -685,13 +802,14 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result
-  final_Group_data_PEV_mean <- subset(total_Group_data_PEV_mean2, select = -row)
+  final_Group_data_PEV_mean1 <- subset(total_Group_data_PEV_mean2, select = -row)
+  final_Group_data_PEV_mean <- grouping_result(final_Group_data_PEV_mean1)
   
   ###PEV_median
   #Combining all the above results
   total_pev_median <- plyr::rbind.fill(vsn_knn_PEV_median, vsn_lls_PEV_median, vsn_svd_PEV_median, 
-                                     loess_knn_PEV_median, loess_lls_PEV_median, loess_svd_PEV_median, 
-                                     rlr_knn_PEV_median, rlr_lls_PEV_median, rlr_svd_PEV_median)
+                                       loess_knn_PEV_median, loess_lls_PEV_median, loess_svd_PEV_median, 
+                                       rlr_knn_PEV_median, rlr_lls_PEV_median, rlr_svd_PEV_median)
   
   #Separating the results groupwise
   total_Group_data_PEV_median <- total_pev_median %>%
@@ -711,14 +829,15 @@ best_combination <- function (data_input, groups){
                        names_vary = "slowest")%>%
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
-  #Final result 
-  final_Group_data_PEV_median <- subset(total_Group_data_PEV_median2, select = -row)
+  #Final result
+  final_Group_data_PEV_median1 <- subset(total_Group_data_PEV_median2, select = -row)
+  final_Group_data_PEV_median <- grouping_result(final_Group_data_PEV_median1)
   
   ###PEV_sd
   #Combining all the above results
   total_pev_sd <- plyr::rbind.fill(vsn_knn_PEV_sd, vsn_lls_PEV_sd, vsn_svd_PEV_sd, 
-                                     loess_knn_PEV_sd, loess_lls_PEV_sd, loess_svd_PEV_sd, 
-                                     rlr_knn_PEV_sd, rlr_lls_PEV_sd, rlr_svd_PEV_sd)
+                                   loess_knn_PEV_sd, loess_lls_PEV_sd, loess_svd_PEV_sd, 
+                                   rlr_knn_PEV_sd, rlr_lls_PEV_sd, rlr_svd_PEV_sd)
   
   #Separating the results groupwise
   total_Group_data_PEV_sd <- total_pev_sd %>%
@@ -739,7 +858,8 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result
-  final_Group_data_PEV_sd <- subset(total_Group_data_PEV_sd2, select = -row)
+  final_Group_data_PEV_sd1 <- subset(total_Group_data_PEV_sd2, select = -row)
+  final_Group_data_PEV_sd <- grouping_result(final_Group_data_PEV_sd1)
   
   #Overall PEV function
   Total_data_PEV = function(data1){
@@ -788,7 +908,8 @@ best_combination <- function (data_input, groups){
   total_pev_overall_mean2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pev_overall_mean <-total_pev_overall_mean2%>%dplyr::slice_min(PEV_mean, n=1, with_ties = TRUE)
+  total_pev_overall_mean1 <-total_pev_overall_mean2%>%dplyr::slice_min(PEV_mean, n=1, with_ties = TRUE)
+  total_pev_overall_mean <- grouping_result(total_pev_overall_mean1)
   original_cols <- c("Overall_Type.PEV_mean", "Overall_value.PEV_mean")
   colnames(total_pev_overall_mean) <- original_cols
   
@@ -824,7 +945,8 @@ best_combination <- function (data_input, groups){
   total_pev_overall_median2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pev_overall_median <-total_pev_overall_median2%>%dplyr::slice_min(PEV_median, n=1, with_ties = TRUE)
+  total_pev_overall_median1 <-total_pev_overall_median2%>%dplyr::slice_min(PEV_median, n=1, with_ties = TRUE)
+  total_pev_overall_median <- grouping_result(total_pev_overall_median1)
   original_cols <- c("Overall_Type.PEV_median", "Overall_value.PEV_median")
   colnames(total_pev_overall_median) <- original_cols
   
@@ -860,7 +982,8 @@ best_combination <- function (data_input, groups){
   total_pev_overall_sd2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pev_overall_sd <-total_pev_overall_sd2%>%dplyr::slice_min(PEV_sd, n=1, with_ties = TRUE)
+  total_pev_overall_sd1 <-total_pev_overall_sd2%>%dplyr::slice_min(PEV_sd, n=1, with_ties = TRUE)
+  total_pev_overall_sd <- grouping_result(total_pev_overall_sd1)
   original_cols <- c("Overall_Type.PEV_sd", "Overall_value.PEV_sd")
   colnames(total_pev_overall_sd) <- original_cols
   
@@ -877,13 +1000,21 @@ best_combination <- function (data_input, groups){
   # Remove even-numbered columns
   result_PEV_names1 <- result_PEV[, -col_indices]
   n <- matrix(t(result_PEV_names1), ncol=1)
-  all_elements <- unlist(n)
+  
+  # Split the values by comma and remove extra spaces
+  split_data <- strsplit(n, ",\\s*")
+  
+  # Flatten the list into a single vector
+  separated_data <- unlist(split_data)
   
   # Get the frequency of each element in the dataframe
-  freq <- table(all_elements)
+  freq <- table(separated_data)
   
-  # Find the most occurring element
-  PEV_best_combination <- names(freq)[which.max(freq)]
+  # Find the maximum frequency
+  max_freq <- max(freq)
+  
+  # Get the names of all elements that have the maximum frequency
+  PEV_best_combination <- names(freq)[freq == max_freq]
   
   #Groupwise PMAD function
   Group_data_PMAD = function(data1, groups){
@@ -951,8 +1082,8 @@ best_combination <- function (data_input, groups){
   ###PMAD_mean
   #Combining all the above results
   total_pmad_mean <- plyr::rbind.fill(vsn_knn_PMAD_mean, vsn_lls_PMAD_mean, vsn_svd_PMAD_mean, 
-                                     loess_knn_PMAD_mean, loess_lls_PMAD_mean, loess_svd_PMAD_mean, 
-                                     rlr_knn_PMAD_mean, rlr_lls_PMAD_mean, rlr_svd_PMAD_mean)
+                                      loess_knn_PMAD_mean, loess_lls_PMAD_mean, loess_svd_PMAD_mean, 
+                                      rlr_knn_PMAD_mean, rlr_lls_PMAD_mean, rlr_svd_PMAD_mean)
   
   #Separating the results groupwise
   total_Group_data_PMAD_mean <- total_pmad_mean %>%
@@ -973,13 +1104,14 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result
-  final_Group_data_PMAD_mean <- subset(total_Group_data_PMAD_mean2, select = -row)
+  final_Group_data_PMAD_mean1 <- subset(total_Group_data_PMAD_mean2, select = -row)
+  final_Group_data_PMAD_mean <- grouping_result(final_Group_data_PMAD_mean1)
   
   ###PMAD_median
   #Combining all the above results
   total_pmad_median <- plyr::rbind.fill(vsn_knn_PMAD_median, vsn_lls_PMAD_median, vsn_svd_PMAD_median, 
-                                      loess_knn_PMAD_median, loess_lls_PMAD_median, loess_svd_PMAD_median, 
-                                      rlr_knn_PMAD_median, rlr_lls_PMAD_median, rlr_svd_PMAD_median)
+                                        loess_knn_PMAD_median, loess_lls_PMAD_median, loess_svd_PMAD_median, 
+                                        rlr_knn_PMAD_median, rlr_lls_PMAD_median, rlr_svd_PMAD_median)
   
   #Separating the results groupwise
   total_Group_data_PMAD_median <- total_pmad_median %>%
@@ -1000,13 +1132,14 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result 
-  final_Group_data_PMAD_median <- subset(total_Group_data_PMAD_median2, select = -row)
+  final_Group_data_PMAD_median1 <- subset(total_Group_data_PMAD_median2, select = -row)
+  final_Group_data_PMAD_median <- grouping_result(final_Group_data_PMAD_median1)
   
   ###PMAD_sd
   #Combining all the above results
   total_pmad_sd <- plyr::rbind.fill(vsn_knn_PMAD_sd, vsn_lls_PMAD_sd, vsn_svd_PMAD_sd, 
-                                      loess_knn_PMAD_sd, loess_lls_PMAD_sd, loess_svd_PMAD_sd, 
-                                      rlr_knn_PMAD_sd, rlr_lls_PMAD_sd, rlr_svd_PMAD_sd)
+                                    loess_knn_PMAD_sd, loess_lls_PMAD_sd, loess_svd_PMAD_sd, 
+                                    rlr_knn_PMAD_sd, rlr_lls_PMAD_sd, rlr_svd_PMAD_sd)
   
   #Separating the results groupwise
   total_Group_data_PMAD_sd <- total_pmad_sd %>%
@@ -1027,7 +1160,8 @@ best_combination <- function (data_input, groups){
     stats::setNames(nm = sub("(.*)_(.*)", "\\2_\\1", names(.)))
   
   #Final result
-  final_Group_data_PMAD_sd <- subset(total_Group_data_PMAD_sd2, select = -row)
+  final_Group_data_PMAD_sd1 <- subset(total_Group_data_PMAD_sd2, select = -row)
+  final_Group_data_PMAD_sd <- grouping_result(final_Group_data_PMAD_sd1)
   
   #Overall PMAD function
   Total_data_PMAD = function(data1){
@@ -1076,7 +1210,8 @@ best_combination <- function (data_input, groups){
   total_pmad_overall_mean2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pmad_overall_mean <-total_pmad_overall_mean2%>%dplyr::slice_min(PMAD_mean, n=1, with_ties = TRUE)
+  total_pmad_overall_mean1 <-total_pmad_overall_mean2%>%dplyr::slice_min(PMAD_mean, n=1, with_ties = TRUE)
+  total_pmad_overall_mean <- grouping_result(total_pmad_overall_mean1)
   original_cols <- c("Overall_Type.PMAD_mean", "Overall_value.PMAD_mean")
   colnames(total_pmad_overall_mean) <- original_cols
   
@@ -1112,7 +1247,8 @@ best_combination <- function (data_input, groups){
   total_pmad_overall_median2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pmad_overall_median <-total_pmad_overall_median2%>%dplyr::slice_min(PMAD_median, n=1, with_ties = TRUE)
+  total_pmad_overall_median1 <-total_pmad_overall_median2%>%dplyr::slice_min(PMAD_median, n=1, with_ties = TRUE)
+  total_pmad_overall_median <- grouping_result(total_pmad_overall_median1)
   original_cols <- c("Overall_Type.PMAD_median", "Overall_value.PMAD_median")
   colnames(total_pmad_overall_median) <- original_cols
   
@@ -1148,7 +1284,8 @@ best_combination <- function (data_input, groups){
   total_pmad_overall_sd2 <- as.data.frame(plyr::rbind.fill(data1, data2, data3, data4, data5, data6, data7, data8, data9))
   
   #Extract the top combination in overall
-  total_pmad_overall_sd <-total_pmad_overall_sd2%>%dplyr::slice_min(PMAD_sd, n=1, with_ties = TRUE)
+  total_pmad_overall_sd1 <-total_pmad_overall_sd2%>%dplyr::slice_min(PMAD_sd, n=1, with_ties = TRUE)
+  total_pmad_overall_sd <- grouping_result(total_pmad_overall_sd1)
   original_cols <- c("Overall_Type.PMAD_sd", "Overall_value.PMAD_sd")
   colnames(total_pmad_overall_sd) <- original_cols
   
@@ -1165,16 +1302,33 @@ best_combination <- function (data_input, groups){
   # Remove even-numbered columns
   result_PMAD_names1 <- result_PMAD[, -col_indices]
   n <- matrix(t(result_PMAD_names1), ncol=1)
-  all_elements <- unlist(n)
+  # Split the values by comma and remove extra spaces
+  split_data <- strsplit(n, ",\\s*")
+  
+  # Flatten the list into a single vector
+  separated_data <- unlist(split_data)
   
   # Get the frequency of each element in the dataframe
-  freq <- table(all_elements)
+  freq <- table(separated_data)
   
-  # Find the most occurring element
-  PMAD_best_combination <- names(freq)[which.max(freq)]
+  # Find the maximum frequency
+  max_freq <- max(freq)
   
-  #Finding the best combination
-  Best_combinations <- cbind(PCV_best_combination, PEV_best_combination, PMAD_best_combination)
+  # Get the names of all elements that have the maximum frequency
+  PMAD_best_combination <- names(freq)[freq == max_freq]
+  
+  # Collapse the combinations into comma-separated strings
+  PCV_best_combination_str <- paste(PCV_best_combination, collapse = ", ")
+  PEV_best_combination_str <- paste(PEV_best_combination, collapse = ", ")
+  PMAD_best_combination_str <- paste(PMAD_best_combination, collapse = ", ")
+  
+  # Create a one-row data frame (or matrix) for the values
+  Best_combinations <- data.frame(
+    PCV_best_combination = PCV_best_combination_str,
+    PEV_best_combination = PEV_best_combination_str,
+    PMAD_best_combination = PMAD_best_combination_str,
+    stringsAsFactors = FALSE
+  )
   
   #Adding names to table
   Combinations <- c("vsn_knn", "vsn_lls", "vsn_svd",
@@ -1228,8 +1382,42 @@ best_combination <- function (data_input, groups){
   PMAD_table1 <- PMAD_table2 %>% dplyr::select(-tidyselect::contains("Type"))
   PMAD_table <- cbind(Combinations, PMAD_table1)
   
-  result_list <- list("Best combinations" = as.data.frame(Best_combinations), "PCV Result" = PCV_table, "PEV Result" = PEV_table, "PMAD Result" =PMAD_table,
-                      "vsn_data" = cbind(com_data_ID, vsn.dat), "loess_data" = cbind(com_data_ID, loess.dat), "rlr_data" = cbind(com_data_ID, rlr.dat),
+  # Apply imputation methods and handle missing values
+  vsn.knn.dat[is.na(vsn.knn.dat)] <- loess.knn.dat[is.na(rlr.knn.dat)] <- rlr.knn.dat[is.na(rlr.knn.dat)] <- 0
+  vsn.lls.dat[is.na(vsn.lls.dat)] <- loess.lls.dat[is.na(rlr.lls.dat)] <- rlr.lls.dat[is.na(rlr.lls.dat)] <- 0
+  vsn.svd.dat[is.na(vsn.svd.dat)] <- loess.svd.dat[is.na(rlr.svd.dat)] <- rlr.svd.dat[is.na(rlr.svd.dat)] <- 0
+  com_data2[is.na(com_data2)] <- 0
+  
+  #Calculating NRMSE
+  nrmse <- function(ximp, xtrue) {
+    # Convert both inputs to numeric vectors
+    ximp_vector <- as.numeric(as.matrix(ximp))
+    xtrue_vector <- as.numeric(as.matrix(xtrue))
+    
+    # Calculate NRMSE
+    sqrt(mean((ximp_vector - xtrue_vector)^2, na.rm = TRUE) / stats::var(xtrue_vector, na.rm = TRUE))
+  }  
+  nrmse_vsn_knn <- nrmse(2^vsn.knn.dat, com_data2)
+  nrmse_vsn_lls <- nrmse(2^vsn.lls.dat, com_data2)
+  nrmse_vsn_svd <- nrmse(2^vsn.svd.dat, com_data2)
+  nrmse_loess_knn <- nrmse(2^loess.knn.dat, com_data2)
+  nrmse_loess_lls <- nrmse(2^loess.lls.dat, com_data2)
+  nrmse_loess_svd <- nrmse(2^loess.svd.dat, com_data2)
+  nrmse_rlr_knn <- nrmse(2^rlr.knn.dat, com_data2)
+  nrmse_rlr_lls <- nrmse(2^rlr.lls.dat, com_data2)
+  nrmse_rlr_svd <- nrmse(2^rlr.svd.dat, com_data2)
+  
+  nrmse <- c(nrmse_vsn_knn, nrmse_vsn_lls, nrmse_vsn_svd,
+             nrmse_loess_knn, nrmse_loess_lls, nrmse_loess_svd,
+             nrmse_rlr_knn, nrmse_rlr_lls, nrmse_rlr_svd)
+  
+  nrmse_1 <- as.data.frame(round(nrmse, digits = 5))
+  
+  nrmse_result <- cbind(Combinations, nrmse_1)
+  colnames(nrmse_result) <- c("Combinations", "NRMSE")  
+  
+  result_list <- list("Best combinations" = as.data.frame(Best_combinations), "PCV Result" = PCV_table, "PEV Result" = PEV_table, "PMAD Result" =PMAD_table, "NRMSE Result" = nrmse_result,
+                      "rollup_protein" = rollup_protein, "vsn_data" = cbind(com_data_ID, vsn.dat), "loess_data" = cbind(com_data_ID, loess.dat), "rlr_data" = cbind(com_data_ID, rlr.dat),
                       "vsn_knn_data" = cbind(com_data_ID,vsn.knn.dat),  "vsn_lls_data" = cbind(com_data_ID,vsn.lls.dat),"vsn_svd_data" = cbind(com_data_ID,vsn.svd.dat),
                       "loess_knn_data" = cbind(com_data_ID, as.data.frame(loess.knn.dat)), "loess_lls_data" =  cbind(com_data_ID, as.data.frame(loess.lls.dat)), "loess_svd_data" = cbind(com_data_ID, as.data.frame(loess.svd.dat)),
                       "rlr_knn_data" =  cbind(com_data_ID, as.data.frame(rlr.knn.dat)), "rlr_lls_data" =  cbind(com_data_ID, as.data.frame(rlr.lls.dat)),"rlr_svd_data" =  cbind(com_data_ID, as.data.frame(rlr.svd.dat)))
